//!
//!
use crate::repositories::{
  bank::models::Bank, member::schema::MemberState, member_class::models::MemberClass,
  member_game_setting::models::MemberGameSetting, member_level::models::MemberLevel,
  member_settlement_setting::models::MemberSettlementSetting, site::models::Site,
};
use diesel::deserialize::QueryableByName;

///
#[derive(PartialEq, PartialOrd, Debug, Clone)]
pub struct _MemberModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub site: Site,
  ///
  pub member_class: MemberClass,
  ///
  pub member_level: MemberLevel,
  ///
  pub bank_account: Option<MemberBankAccountModel>,
  ///
  pub username: String,
  ///
  pub password: String,
  ///
  pub nickname: String,
  ///
  pub parent_member_id: Option<uuid::Uuid>,
  ///
  pub child_member_count: i64,
  ///
  pub mobile_phone_number: Option<String>,
  ///
  pub state: MemberState,
  ///
  pub state_changed_at: Option<i64>,
  ///
  pub member_game_setting: Option<MemberGameSetting>,
  ///
  pub member_settlement_setting: Option<MemberSettlementSetting>,
  ///
  pub last_signined_ip: Option<String>,
  ///
  pub last_signined_at: Option<i64>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
  ///
  pub deleted_at: Option<i64>,
}

impl QueryableByName<diesel::pg::Pg> for _MemberModel {
  fn build<R: diesel::row::NamedRow<diesel::pg::Pg>>(row: &R) -> diesel::deserialize::Result<Self> {
    let site = Site {
      id: row.get("s_id")?,
      url: row.get("s_url")?,
      name: row.get("s_name")?,
      path: row.get("s_path")?,
      show: row.get("s_show")?,
      can_use: row.get("s_can_use")?,
      memo: row.get("s_memo")?,
      expires_at: row.get("s_expires_at")?,
      created_at: row.get("s_created_at")?,
      updated_at: row.get("s_updated_at")?,
    };

    let member_class = MemberClass {
      id: row.get("mc_id")?,
      parent_id: row.get("mc_parent_id")?,
      name: row.get("mc_name")?,
      show: row.get("mc_show")?,
      created_at: row.get("mc_created_at")?,
      updated_at: row.get("mc_updated_at")?,
    };

    let member_level = MemberLevel {
      id: row.get("ml_id")?,
      name: row.get("ml_name")?,
      show: row.get("ml_show")?,
      sort_order: row.get("ml_sort_order")?,
      created_at: row.get("ml_created_at")?,
      updated_at: row.get("ml_updated_at")?,
    };

    let member_game_setting = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("mgs_id")?
    {
      Some(_) => Some(MemberGameSetting {
        id: row.get("mgs_id")?,
        member_id: row.get("mgs_member_id")?,
        can_bet_casino: row.get("mgs_can_bet_casino")?,
        can_bet_slot: row.get("mgs_can_bet_slot")?,
        can_bet_powerball: row.get("mgs_can_bet_powerball")?,
        can_bet_powerladder: row.get("mgs_can_bet_powerladder")?,
        can_bet_eos: row.get("mgs_can_bet_eos")?,
        can_bet_bogglepowerball: row.get("mgs_can_bet_bogglepowerball")?,
        created_at: row.get("mgs_created_at")?,
        updated_at: row.get("mgs_updated_at")?,
      }),
      None => None,
    };

    let member_settlement_setting = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("mss_id")?
    {
      Some(_) => Some(MemberSettlementSetting {
        id: row.get("mss_id")?,
        member_id: row.get("mss_member_id")?,
        can_exchange: row.get("mss_can_exchange")?,
        can_first_charge_comp: row.get("mss_can_first_charge_comp")?,
        can_per_charge_comp: row.get("mss_can_per_charge_comp")?,
        manual_payment_type_for_partner: row.get("mss_manual_payment_type_for_partner")?,
        settlement_type: row.get("mss_settlement_type")?,
        rate_casino: row.get("mss_rate_casino")?,
        rate_casino_loosing: row.get("mss_rate_casino_loosing")?,
        rate_casino_bacara: row.get("mss_rate_casino_bacara")?,
        rate_casino_roulette: row.get("mss_rate_casino_roulette")?,
        rate_casino_dragon_tiger: row.get("mss_rate_casino_dragon_tiger")?,
        rate_slot: row.get("mss_rate_slot")?,
        rate_slot_loosing: row.get("mss_rate_slot_loosing")?,
        rate_powerball_single: row.get("mss_rate_powerball_single")?,
        rate_powerball_combo: row.get("mss_rate_powerball_combo")?,
        rate_powerladder_single: row.get("mss_rate_powerladder_single")?,
        rate_powerladder_combo: row.get("mss_rate_powerladder_combo")?,
        rate_eos_single: row.get("mss_rate_eos_single")?,
        rate_eos_combo: row.get("mss_rate_eos_combo")?,
        rate_bogglepowerball_single: row.get("mss_rate_bogglepowerball_single")?,
        rate_bogglepowerball_combo: row.get("mss_rate_bogglepowerball_combo")?,
        created_at: row.get("mss_created_at")?,
        updated_at: row.get("mss_updated_at")?,
      }),
      None => None,
    };

    let member_bank_account = match row
      .get::<diesel::sql_types::Nullable<diesel::sql_types::Uuid>, Option<uuid::Uuid>>("mba_id")?
    {
      Some(_) => {
        let bank = Bank {
          id: row.get("mba_b_id")?,
          name: row.get("mba_b_name")?,
          sort_order: row.get("mba_b_sort_order")?,
          show: row.get("mba_b_show")?,
          can_use: row.get("mba_b_can_use")?,
          memo: row.get("mba_b_memo")?,
          created_at: row.get("mba_b_created_at")?,
          updated_at: row.get("mba_b_updated_at")?,
        };

        Some(MemberBankAccountModel {
          id: row.get("mba_id")?,
          member_id: row.get("mba_member_id")?,
          bank,
          name: row.get("mba_name")?,
          account_number: row.get("mba_account_number")?,
          exchange_password: row.get("mba_exchange_password")?,
          memo: row.get("mba_memo")?,
          created_at: row.get("mba_created_at")?,
          updated_at: row.get("mba_updated_at")?,
        })
      }
      None => None,
    };

    Ok(_MemberModel {
      id: row.get("m_id")?,
      site,
      member_class,
      member_level,
      username: row.get("m_username")?,
      password: row.get("m_password")?,
      nickname: row.get("m_nickname")?,
      mobile_phone_number: row.get("m_mobile_phone_number")?,
      state: row.get("m_state")?,
      state_changed_at: row.get("m_state_changed_at")?,
      member_game_setting,
      member_settlement_setting,
      bank_account: member_bank_account,
      parent_member_id: row.get("m_parent_member_id")?,
      child_member_count: row.get("m_child_member_count")?,
      last_signined_ip: row.get("m_last_signined_ip")?,
      last_signined_at: row.get("m_last_signined_at")?,
      created_at: row.get("m_created_at")?,
      updated_at: row.get("m_updated_at")?,
      deleted_at: row.get("m_deleted_at")?,
    })
  }
}

///
#[derive(PartialEq, PartialOrd, Debug, Clone)]
pub struct MemberModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub site: Site,
  ///
  pub member_class: MemberClass,
  ///
  pub member_level: MemberLevel,
  ///
  pub bank_account: Option<MemberBankAccountModel>,
  ///
  pub username: String,
  ///
  pub password: String,
  ///
  pub nickname: String,
  ///
  pub parent_member: Option<Box<MemberModel>>,
  ///
  pub child_member_count: i64,
  ///
  pub mobile_phone_number: Option<String>,
  ///
  pub state: MemberState,
  ///
  pub state_changed_at: Option<i64>,
  ///
  pub member_game_setting: Option<MemberGameSetting>,
  ///
  pub member_settlement_setting: Option<MemberSettlementSetting>,
  ///
  pub last_signined_ip: Option<String>,
  ///
  pub last_signined_at: Option<i64>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
  ///
  pub deleted_at: Option<i64>,
}

pub fn from_member_model(_m: &_MemberModel, p: Option<MemberModel>) -> MemberModel {
  MemberModel {
    id: _m.id,
    site: _m.site.clone(),
    member_class: _m.member_class.clone(),
    member_level: _m.member_level.clone(),
    bank_account: _m.bank_account.clone(),
    username: _m.username.clone(),
    password: _m.password.clone(),
    nickname: _m.nickname.clone(),
    parent_member: p.map(Box::new),
    child_member_count: _m.child_member_count,
    mobile_phone_number: _m.mobile_phone_number.clone(),
    state: _m.state,
    state_changed_at: _m.state_changed_at,
    member_game_setting: _m.member_game_setting.clone(),
    member_settlement_setting: _m.member_settlement_setting.clone(),
    last_signined_ip: _m.last_signined_ip.clone(),
    last_signined_at: _m.last_signined_at,
    created_at: _m.created_at,
    updated_at: _m.updated_at,
    deleted_at: _m.deleted_at,
  }
}

#[derive(Eq, Hash, PartialEq, PartialOrd, Debug, Clone)]
pub struct MemberBankAccountModel {
  ///
  pub id: uuid::Uuid,
  ///
  pub member_id: uuid::Uuid,
  ///
  pub bank: Bank,
  ///
  pub name: String,
  ///
  pub account_number: String,
  ///
  pub exchange_password: String,
  ///
  pub memo: Option<String>,
  ///
  pub created_at: i64,
  ///
  pub updated_at: i64,
}
